// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane/fastlane_create_keychain_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane/fastlane_deploy_pilot_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane/fastlane_deploy_supply_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane/fastlane_get_certificates_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/fastlane/fastlane_create_keychain_usecase.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/fastlane/fastlane_deploy_pilot_usecase.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/fastlane/fastlane_deploy_supply_usecase.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/fastlane/fastlane_get_certificates_usecase.dart';

class Fastlane {
  factory Fastlane() => Fastlane._(
        createKeychainUsecase: getIt(),
        deployPilotUsecase: getIt(),
        deploySupplyUsecase: getIt(),
        getCertificateUsecase: getIt(),
      );

  Fastlane._({
    required FastlaneCreateKeychainUsecase createKeychainUsecase,
    required FastlaneDeployPilotUsecase deployPilotUsecase,
    required FastlaneDeploySupplyUsecase deploySupplyUsecase,
    required FastlaneGetCertificateUsecase getCertificateUsecase,
  })  : _createKeychainUsecase = createKeychainUsecase,
        _deployPilotUsecase = deployPilotUsecase,
        _deploySupplyUsecase = deploySupplyUsecase,
        _getCertificateUsecase = getCertificateUsecase;

  final FastlaneCreateKeychainUsecase _createKeychainUsecase;
  final FastlaneDeployPilotUsecase _deployPilotUsecase;
  final FastlaneDeploySupplyUsecase _deploySupplyUsecase;
  final FastlaneGetCertificateUsecase _getCertificateUsecase;

  FutureOrResult<void> createKeychain(
    FastlaneCreateKeychainParameters params,
  ) =>
      _createKeychainUsecase(params);

  FutureOrResult<void> deployAppStore(FastlaneDeployPilotParameters params) =>
      _deployPilotUsecase(params);

  FutureOrResult<void> deployGooglePlay(
    FastlaneDeploySupplyParameters params,
  ) =>
      _deploySupplyUsecase(params);

  FutureOrResult<void> getCertificates(
    FastlaneGetCertificatesParameters params,
  ) =>
      _getCertificateUsecase(params);
}
