// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_context.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/utils/authentication_methods.dart';
import 'package:wyatt_http_client/src/utils/header_keys.dart';

/// {@template basic_auth_middleware}
/// A middleware that adds basic authentication to the request.
/// {@endtemplate}
class BasicAuthMiddleware with OnRequestMiddleware implements Middleware {
  /// {@macro basic_auth_middleware}
  const BasicAuthMiddleware({
    this.username,
    this.password,
    this.authenticationHeader = HeaderKeys.authorization,
  });

  /// The username to use for authentication.
  final String? username;

  /// The password to use for authentication.
  final String? password;

  /// The header to use for authentication.
  final String authenticationHeader;

  @override
  String getName() => 'BasicAuth';

  @override
  Future<MiddlewareRequest> onRequest(
    MiddlewareContext context,
    MiddlewareRequest request,
  ) async {
    if (username == null || password == null) {
      return request;
    }

    final mutation = {
      authenticationHeader: '${AuthenticationMethods.basic} '
          '${base64Encode(utf8.encode('$username:$password'))}',
    };
    final Map<String, String> headers = request.headers..addAll(mutation);
    request.modifyRequest(request.unfreezedRequest.copyWith(headers: headers));
    return request;
  }
}
