import 'package:flutter/material.dart';

/// {@template card_background}
/// A widget that displays a background for a card.
/// {@endtemplate}
class CardBackground extends StatefulWidget {
  /// {@macro card_background}
  const CardBackground({super.key, this.background, this.cardKey});

  /// Background to display
  final Widget? background;

  /// Key of the card to display the background for
  final GlobalKey? cardKey;

  @override
  State<CardBackground> createState() => _CardBackgroundState();
}

class _CardBackgroundState extends State<CardBackground> {
  Size _cardSize = Size.zero;

  @override
  void initState() {
    super.initState();
    if (widget.background != null) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _resizeCard();
      });
    }
  }

  @override
  void didUpdateWidget(covariant CardBackground oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.background != null) {
      _resizeCard();
    }
  }

  void _resizeCard() {
    final RenderObject? renderBox =
        widget.cardKey?.currentContext?.findRenderObject();
    if (renderBox != null) {
      setState(() {
        _cardSize =
            Size(renderBox.paintBounds.width, renderBox.paintBounds.height);
      });
    }
  }

  @override
  Widget build(BuildContext context) => SizedBox(
        width: _cardSize.width,
        height: _cardSize.height,
        child: Center(child: widget.background),
      );
}
