// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/core/design_system/colors.dart';

class FlatButtonThemeExtensionImpl extends FlatButtonThemeExtension {
  const FlatButtonThemeExtensionImpl({
    super.disabledStyle,
    super.focusedStyle,
    super.hoveredStyle,
    super.normalStyle,
    super.tappedStyle,
  });

  factory FlatButtonThemeExtensionImpl.light({ThemeData? theme}) {
    theme ??= ThemeData.light();
    final style = FlatButtonStyle(
      labelStyle: theme.textTheme.labelLarge,
      radius: BorderRadius.circular(15),
      padding: const EdgeInsets.all(10),
      foregroundColors: WyattColors.blueBtnGradient,
      backgroundColors: const MultiColor.single(WyattColors.light),
      borderColors: WyattColors.blueBtnGradient,
      stroke: 3,
      animationDuration: const Duration(milliseconds: 150),
    );
    return FlatButtonThemeExtensionImpl(
      normalStyle: style,
      disabledStyle: style.copyWith(
        labelStyle: theme.textTheme.labelLarge?.copyWith(
          color: WyattColors.blue1.withOpacity(0.4),
        ),
        foregroundColors: MultiColor.single(WyattColors.blue1.withOpacity(0.4)),
        backgroundColors: const MultiColor.single(WyattColors.light),
        borderColors: MultiColor.single(WyattColors.blue1.withOpacity(0.4)),
        stroke: 1,
      ),
      hoveredStyle: style.copyWith(
        labelStyle: theme.textTheme.labelLarge?.copyWith(
          color: WyattColors.light,
        ),
        foregroundColors: const MultiColor.single(WyattColors.light),
        backgroundColors: WyattColors.blueBtnHoverGradient,
        borderColors: WyattColors.blueBtnHoverGradient,
      ),
      focusedStyle: style.copyWith(
        stroke: 5,
        borderColors: WyattColors.blueBtnFocusGradient,
      ),
      tappedStyle: style.copyWith(
        labelStyle: theme.textTheme.labelLarge?.copyWith(
          color: WyattColors.light,
        ),
        foregroundColors: const MultiColor.single(WyattColors.light),
        backgroundColors: WyattColors.darkBlueGradient,
        borderColors: WyattColors.darkBlueGradient,
      ),
    );
  }

  factory FlatButtonThemeExtensionImpl.dark({ThemeData? theme}) {
    theme ??= ThemeData.dark();
    final style = FlatButtonStyle(
      labelStyle: theme.textTheme.labelLarge,
      radius: BorderRadius.circular(15),
      padding: const EdgeInsets.all(10),
      foregroundColors: const MultiColor.single(WyattColors.light),
      backgroundColors: const MultiColor.single(WyattColors.black),
      borderColors: WyattColors.blueBtnGradient,
      stroke: 3,
      animationDuration: const Duration(milliseconds: 150),
    );
    return FlatButtonThemeExtensionImpl(
      normalStyle: style,
      disabledStyle: style.copyWith(
        labelStyle: theme.textTheme.labelLarge?.copyWith(
          color: WyattColors.gray1,
        ),
        foregroundColors: const MultiColor.single(WyattColors.gray1),
        backgroundColors:
            MultiColor.single(WyattColors.light.withOpacity(0.04)),
        borderColors: WyattColors.grayGradient,
        stroke: 1,
      ),
      hoveredStyle: style.copyWith(
        foregroundColors: const MultiColor.single(WyattColors.light),
        backgroundColors: WyattColors.blueBtnHoverGradient,
        borderColors: WyattColors.blueBtnHoverGradient,
      ),
      focusedStyle: style.copyWith(
        stroke: 5,
        borderColors: WyattColors.blueBtnFocusGradient,
      ),
      tappedStyle: style.copyWith(
        foregroundColors: const MultiColor.single(WyattColors.light),
        backgroundColors: WyattColors.darkBlueGradient,
        borderColors: WyattColors.darkBlueGradient,
      ),
    );
  }
}
